(function() {
  var NoRepeatSprite, array_max, compile_css, compile_norepeat_sprite, compile_png, composition_dimension, compute_composition, compute_layout, css_class, curry, ends_by_png, fs, gdlib, isDirSync, keys, path, serial_call_chain, serial_map_async, tumbler, values;
  var __slice = Array.prototype.slice, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; }, __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  };
  gdlib = require('node-gd');
  fs = require('fs');
  path = require('path');
  compute_layout = (require('./layout')).compute_layout;
  serial_call_chain = function(operations, success, failure) {
    var head, process_tail, tail;
    if (success == null) {
      success = (function() {});
    }
    if (failure == null) {
      failure = (function() {});
    }
    if (operations.length === 0) {
      return success();
    } else {
      head = operations[0];
      tail = operations.slice(1);
      process_tail = function() {
        return serial_call_chain(tail, success, failure);
      };
      return head(process_tail, failure);
    }
  };
  serial_map_async = function(arr, f, callback) {
    var aux, res;
    if (callback == null) {
      callback = function() {};
    }
    res = [];
    aux = function(arr) {
      if (arr.length === 0) {
        return callback(res);
      } else {
        return f(arr[0], function(ft) {
          res.push(ft);
          return aux(arr.slice(1));
        });
      }
    };
    return aux(arr);
  };
  values = function(c) {
    var k, v, _results;
    _results = [];
    for (k in c) {
      v = c[k];
      _results.push(v);
    }
    return _results;
  };
  keys = function(c) {
    var k, v, _results;
    _results = [];
    for (k in c) {
      v = c[k];
      _results.push(k);
    }
    return _results;
  };
  array_max = function(arr) {
    return Math.max.apply(null, arr);
  };
  curry = function() {
    var f, fixed;
    f = arguments[0], fixed = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    return function() {
      var extra_args;
      extra_args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return f.apply(null, fixed.concat(extra_args));
    };
  };
  composition_dimension = function(image_positions) {
    var H, W, pos, positions;
    positions = values(image_positions);
    W = array_max((function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = positions.length; _i < _len; _i++) {
        pos = positions[_i];
        _results.push(pos.x + pos.w);
      }
      return _results;
    })());
    H = array_max((function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = positions.length; _i < _len; _i++) {
        pos = positions[_i];
        _results.push(pos.y + pos.h);
      }
      return _results;
    })());
    return [W, H];
  };
  compute_composition = function(image_positions, callback) {
    var H, W, append_picture, composition, imgPath, position, spriteInfo, _ref;
    _ref = composition_dimension(image_positions), W = _ref[0], H = _ref[1];
    composition = gdlib.createTrueColor(W, H);
    append_picture = function(spriteInfo, callback) {
      var img, imgPath, position;
      position = spriteInfo.position;
      imgPath = spriteInfo.imgPath;
      img = gdlib.createFromPng(imgPath);
      img.copy(composition, position.x, position.y, 0, 0, position.w, position.h);
      return callback();
    };
    spriteInfo = (function() {
      var _results;
      _results = [];
      for (imgPath in image_positions) {
        position = image_positions[imgPath];
        _results.push({
          "imgPath": imgPath,
          "position": position
        });
      }
      return _results;
    })();
    return serial_map_async(spriteInfo, append_picture, function() {
      return callback(composition);
    });
  };
  compile_png = function(sprite_image_path, image_positions, success, failure) {
    return compute_composition(image_positions, function(composition) {
      composition.savePng(sprite_image_path);
      return success();
    });
  };
  css_class = function(css_classname, sprite_image_path, box) {
    return "." + css_classname + " {\n    background-repeat: none;\n    background-image: url(" + sprite_image_path + ");\n    background-position: -" + box.x + "px  -" + box.y + "px ;\n    width: " + box.w + "px;\n    height: " + box.h + "px;\n}\n";
  };
  compile_css = function(css_path, sprite_image_path, image_positions, success, failure) {
    var box, css_classname, css_file, image_filepath;
    css_file = fs.createWriteStream(css_path);
    for (image_filepath in image_positions) {
      box = image_positions[image_filepath];
      css_classname = path.basename(image_filepath, ".png");
      css_file.write(css_class(css_classname, sprite_image_path, box));
    }
    return success();
  };
  NoRepeatSprite = (function() {
    function NoRepeatSprite() {
      this.image_dimensions = {};
    }
    NoRepeatSprite.prototype.add = function(filepath, callback) {
      return gdlib.openPng(filepath, __bind(function(err, img) {
        this.image_dimensions[filepath] = [img.width, img.height];
        return callback([img.width, img.height]);
      }, this));
    };
    NoRepeatSprite.prototype.compile = function(prefix, success, failure) {
      var css_task, image_positions, png_task, sprite_basename, sprite_image_path, stylesheet_path;
      if (success == null) {
        success = (function() {});
      }
      if (failure == null) {
        failure = (function() {});
      }
      image_positions = compute_layout(this.image_dimensions);
      sprite_image_path = prefix + ".png";
      sprite_basename = path.basename(sprite_image_path);
      stylesheet_path = prefix + ".css";
      png_task = curry(compile_png, sprite_image_path, image_positions);
      css_task = curry(compile_css, stylesheet_path, sprite_basename, image_positions);
      return serial_call_chain([png_task, css_task], success, failure);
    };
    return NoRepeatSprite;
  })();
  ends_by_png = function(filepath) {
    return filepath.slice(-3) === "png";
  };
  isDirSync = function(dirpath) {
    var stat;
    if (path.existsSync(dirpath)) {
      stat = fs.statSync(dirpath);
      return stat != null ? stat.isDirectory() : void 0;
    } else {
      return false;
    }
  };
  compile_norepeat_sprite = function(dirpath, success, failure) {
    var add_img_async, filepath, filepaths, prefix, relative_file_path, sprite;
    if (success == null) {
      success = (function() {});
    }
    if (failure == null) {
      failure = (function() {});
    }
    prefix = dirpath;
    relative_file_path = fs.readdirSync(dirpath);
    filepaths = (function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = relative_file_path.length; _i < _len; _i++) {
        filepath = relative_file_path[_i];
        _results.push(path.join(dirpath, filepath));
      }
      return _results;
    })();
    filepaths = filepaths.filter(ends_by_png);
    sprite = new NoRepeatSprite();
    add_img_async = function(filepath, callback) {
      return sprite.add(filepath, callback);
    };
    return serial_map_async(filepaths, add_img_async, function() {
      return sprite.compile(dirpath, success, failure);
    });
  };
  try {
    tumbler = require('tumbler');
    module.exports.SpriteCompiler = (function() {
      __extends(SpriteCompiler, tumbler.Compiler);
      function SpriteCompiler() {
        SpriteCompiler.__super__.constructor.apply(this, arguments);
      }
      SpriteCompiler.prototype.match = function(dest) {
        var prefix;
        if (dest.slice(-11) === ".sprite.css") {
          prefix = this.extract_prefix(dest);
          return isDirSync(prefix);
        } else {
          return false;
        }
      };
      SpriteCompiler.prototype.extract_prefix = function(css_filepath) {
        return css_filepath.slice(0, -4);
      };
      SpriteCompiler.prototype.get_source_list = function(dest) {
        var filepath, filepaths, prefix, _i, _len, _results;
        prefix = this.extract_prefix(dest);
        filepaths = (fs.readdirSync(prefix)).filter(ends_by_png);
        _results = [];
        for (_i = 0, _len = filepaths.length; _i < _len; _i++) {
          filepath = filepaths[_i];
          _results.push(path.join(prefix, filepath));
        }
        return _results;
      };
      SpriteCompiler.prototype.build = function(dest, success, failure) {
        var sprite_dirpath;
        sprite_dirpath = dest.slice(0, -4);
        return compile_norepeat_sprite(sprite_dirpath, success, failure);
      };
      return SpriteCompiler;
    })();
  } catch (exception) {

  }
  module.exports.NoRepeatSprite = NoRepeatSprite;
  module.exports.compile_norepeat_sprite = compile_norepeat_sprite;
}).call(this);
