gdlib = require 'node-gd'
fs = require 'fs'
path = require 'path'
compute_layout = (require './layout').compute_layout



serial_call_chain = (operations, success=(->), failure=(->))->
    if operations.length == 0
        success()
    else
        head = operations[0]
        tail = operations[1..]
        process_tail = ->
            serial_call_chain tail, success, failure
        head process_tail,failure


serial_map_async = (arr,f,callback=->)->
    res = []
    aux = (arr)->
        if arr.length == 0
            callback res
        else
            f arr[0], (ft)->
                res.push ft
                aux arr[1..]
    aux arr

values = (c)->
    ( v for k,v of c )

keys = (c)->
    ( k for k,v of c )

array_max = (arr)->
    Math.max.apply null, arr

curry = (f,fixed...)->
    (extra_args...)->
        f.apply(null,(fixed.concat extra_args))
        

composition_dimension = (image_positions)->
    positions = values image_positions
    W = array_max ( pos.x + pos.w for pos in positions )
    H = array_max ( pos.y + pos.h for pos in positions )
    [W, H]

compute_composition = (image_positions, callback)->
    [W,H] = composition_dimension image_positions
    composition = gdlib.createTrueColor W,H
    append_picture = (spriteInfo, callback)->
        position = spriteInfo.position
        imgPath = spriteInfo.imgPath
        img = gdlib.createFromPng imgPath
        img.copy composition, position.x, position.y, 0, 0, position.w, position.h
        callback()
    spriteInfo = ( {"imgPath":imgPath,"position":position} for imgPath,position of image_positions )
    serial_map_async spriteInfo, append_picture, ->callback composition

compile_png = (sprite_image_path, image_positions, success, failure) ->
    compute_composition image_positions, (composition)->
        composition.savePng sprite_image_path
        success()

css_class = (css_classname, sprite_image_path, box)->
    """
    .#{ css_classname } {
        background-repeat: none;
        background-image: url(#{sprite_image_path});
        background-position: -#{box.x}px  -#{box.y}px ;
        width: #{box.w}px;
        height: #{box.h}px;
    }

    """

compile_css = (css_path, sprite_image_path, image_positions, success, failure) ->
    css_file = fs.createWriteStream css_path
    for image_filepath, box of image_positions
        css_classname  = path.basename image_filepath, ".png"
        css_file.write css_class css_classname,sprite_image_path, box
    success()


class NoRepeatSprite
    
    constructor: ->
        @image_dimensions = {}
    
    add: (filepath,callback)->
        gdlib.openPng filepath,(err, img)=>
            @image_dimensions[filepath] = [img.width, img.height]
            callback [img.width, img.height]
    
    compile: (prefix, success=(->), failure=(->))->
        image_positions = compute_layout @image_dimensions
        sprite_image_path = prefix+".png"
        sprite_basename = path.basename sprite_image_path
        stylesheet_path =  prefix+".css"
        png_task = curry compile_png, sprite_image_path, image_positions
        css_task = curry compile_css, stylesheet_path, sprite_basename, image_positions
        serial_call_chain [png_task, css_task], success, failure


ends_by_png = (filepath)->
    filepath[-3..] == "png"

isDirSync= (dirpath)->
    if path.existsSync dirpath
        stat = fs.statSync dirpath
        stat?.isDirectory()
    else
        false
    

compile_norepeat_sprite = (dirpath, success=(->), failure=(->))->
    prefix = dirpath
    relative_file_path = fs.readdirSync dirpath
    filepaths = ( path.join(dirpath,filepath) for filepath in relative_file_path )
    filepaths = filepaths.filter ends_by_png
    sprite = new NoRepeatSprite()

    add_img_async = (filepath,callback)->
        sprite.add filepath,callback
    
    serial_map_async filepaths, add_img_async, ->
        sprite.compile dirpath, success, failure

try    
    tumbler = require 'tumbler'

    class module.exports.SpriteCompiler extends tumbler.Compiler

        match: (dest)->
            # Returns true if the destination
            # can actually be built by this compiler.
            if dest[-11..] == ".sprite.css"
                prefix = @extract_prefix dest
                isDirSync prefix
            else
                false
                
        extract_prefix: (css_filepath)->
            css_filepath[...-4]
        
        get_source_list: (dest)->
            prefix = @extract_prefix dest
            filepaths = (fs.readdirSync prefix).filter ends_by_png
            ( (path.join prefix,filepath) for filepath in filepaths )
        
        build: (dest, success, failure)->
            sprite_dirpath = dest[...-4]
            compile_norepeat_sprite sprite_dirpath, success, failure
   
catch exception
    # standalone.

module.exports.NoRepeatSprite = NoRepeatSprite
module.exports.compile_norepeat_sprite = compile_norepeat_sprite
